! This file is part of HiggsBounds
!  -KW
!******************************************************************
module S95tables
!******************************************************************	
 use S95tables_type1	
 use S95tables_type2
 implicit none
 private 
 public :: nLEPtable1,nLEPtable2,nTEVtable1,nTEVtable2, &
         & S95_t1,S95_t2, &
	 & setup_S95tables,deallocate_S95tables, &
	 & calcfact,outputproc, &
	 & check_against_bound
 
 integer :: nLEPtable1,nLEPtable2,nTEVtable1,nTEVtable2
 
  ! to turn this feature off, set deltaMh_TEV and/or deltaMh_LEP to a very small value
 double precision, parameter :: deltaMh_LEP=2.0D0  
 double precision, parameter :: deltaMh_TEV=10.0D0 
 ! We would recommend that you do not increase these values
 ! nb. deltaMh_LEP is only used for processes involving only one Higgs at the moment

 !double precision, parameter :: deltaMh_LEP=10.0D0 !crazy values - for debugging only 
 !double precision, parameter :: deltaMh_TEV=15.0D0 !crazy(ish) values - for debugging only

 ! eps determines how strict the Standard Model test is 
 double precision, parameter :: eps=2.0D-2
 !double precision, parameter :: eps=1.0D3 !crazy value - for debugging only

 !table type 1-----------------------------	
 type(table1),allocatable :: S95_t1(:)
 !------------------------------------------
 
 !table type 2------------------------------
 type(table2),allocatable :: S95_t2(:)					
 !------------------------------------------				
 
 contains
 
 !**********************************************************
 subroutine setup_S95tables
 ! Allocates  and calls subroutines to fill S95_t1, S95_t2 
 ! (which store the experimental data)
 ! Sets deltaMh_TEV,deltaMh_LEP which govern how close Higgs
 ! need to be in mass before HiggsBounds combines their cross sections
 !**********************************************************
  use usefulbits, only : debug, &
        &                TEVt1Mhmax,TEVt1Mhmin,LEPt12Mhmax,LEPt12Mhmin, &
        &                TEVrange_min,TEVrange_max,LEPrange_min,LEPrange_max
  use theory_BRfunctions, only : BRSMt1Mhmax,BRSMt1Mhmin
  use theory_XS_SM_functions, only : XS_SM_functions_xmin,XS_SM_functions_xmax
  implicit none	
  !-----------------------------------internal 
  integer :: i
  double precision :: max_tev_deltaMh,max_lep_deltaMh
  !-------------------------------------------	

  ! these numbers have to be changed appropriately every time a table is added
  ! or taken away:  
  nLEPtable1=4                 ! table type 1 involves 1 Higgs
  nTEVtable1=36	  
  nLEPtable2=9                 ! table type 2 involves 2 Higgs
  nTEVtable2=0   

  allocate(S95_t1(nLEPtable1+nTEVtable1))		
  allocate(S95_t2(nLEPtable2+nTEVtable2))	

  call initializetables_type1_blank(S95_t1)
  call initializetables_type2_blank(S95_t2)

  call initializetables1(S95_t1)
  call initializetables2(S95_t2)	
  
  ! checks that the numbers of LEP and TEV tables set in S95_t1   
  ! corresponds to the number set here
  if(   (count(S95_t1%expt.eq.'LEP').ne.nLEPtable1) &
    .or.(count(S95_t1%expt.eq.'CDF')+count(S95_t1%expt.eq.' D0')+count(S95_t1%expt.eq.'TCB').ne.nTEVtable1))then
   stop 'error in setup_S95tables (a)'
  endif 

  ! checks that the numbers of LEP and TEV tables set in S95_t2   
  ! corresponds to the number set here  
  if(   (count(S95_t2%expt.eq.'LEP').ne.nLEPtable2) &
    .or.(count(S95_t2%expt.eq.'CDF')+count(S95_t2%expt.eq.' D0')+count(S95_t2%expt.eq.'TCB').ne.nTEVtable2))then
   stop 'error in setup_S95tables (b)'
  endif  

  ! checks that none of the id's are repeated
  do i=lbound(S95_t1,dim=1),ubound(S95_t1,dim=1)
   if( count(S95_t1%id.eq.S95_t1(i)%id) &
      +count(S95_t2%id.eq.S95_t1(i)%id).ne.1)then
    stop 'error in setup_S95tables (c)'
   endif
  enddo 
  
  do i=lbound(S95_t2,dim=1),ubound(S95_t2,dim=1)
   if( count(S95_t1%id.eq.S95_t2(i)%id)&
      +count(S95_t2%id.eq.S95_t2(i)%id).ne.1)then
    stop 'error in setup_S95tables (d)'
   endif
  enddo 
  
  ! looks for the min and max values of Mh in the LEP tables
  ! will be used in input.f90 to work out which SM para need to be calculated  
  
  ! initial (impossible) values
  LEPt12Mhmin=1.0D6
  LEPt12Mhmax=0.0D0
  
  do i=lbound(S95_t1,dim=1),ubound(S95_t1,dim=1)
   if( S95_t1(i)%expt.eq.'LEP')then
    if(S95_t1(i)%xmax.gt.LEPt12Mhmax)LEPt12Mhmax=S95_t1(i)%xmax
    if(S95_t1(i)%xmin.lt.LEPt12Mhmin)LEPt12Mhmin=S95_t1(i)%xmin  
   endif
  enddo  

  do i=lbound(S95_t2,dim=1),ubound(S95_t2,dim=1)
   if( S95_t2(i)%expt.eq.'LEP')then
    if(S95_t2(i)%xmax2.gt.LEPt12Mhmax)LEPt12Mhmax=S95_t2(i)%xmax2
    if(S95_t2(i)%xmin1.lt.LEPt12Mhmin)LEPt12Mhmin=S95_t2(i)%xmin1  
   endif
  enddo  

  ! looks for the min and max values of Mh in the Tevatron tables
  ! will be used in theo_SM.f90 to work out which SM para need to be calculated  
  
  ! initial (impossible) values
  TEVt1Mhmin=1.0D6
  TEVt1Mhmax=0.0D0
  
  do i=lbound(S95_t1,dim=1),ubound(S95_t1,dim=1)
   if( (S95_t1(i)%expt.eq.'CDF') &
   .or.(S95_t1(i)%expt.eq.' D0') &
   .or.(S95_t1(i)%expt.eq.'TCB'))then

    if(S95_t1(i)%xmax.gt.TEVt1Mhmax)TEVt1Mhmax=S95_t1(i)%xmax
    if(S95_t1(i)%xmin.lt.TEVt1Mhmin)TEVt1Mhmin=S95_t1(i)%xmin  
   endif
  enddo

  if(deltaMh_LEP.gt.2.1d0)write(*,*)'WARNING: deltaMh_LEP.gt.2.1d0'
  if(deltaMh_TEV.gt.10.1d0)write(*,*)'WARNING: deltaMh_TEV.gt.10.1d0'
  if(eps.gt.5.0d-1)write(*,*)'WARNING: eps.gt.5.0d-1'

  ! it is possible to set a different deltax foreach table if you like
  do i=lbound(S95_t1,dim=1),ubound(S95_t1,dim=1)
   if( (S95_t1(i)%expt.eq.'CDF') &
   .or.(S95_t1(i)%expt.eq.' D0') &
   .or.(S95_t1(i)%expt.eq.'TCB'))then
      S95_t1(i)%deltax=deltaMh_TEV
   else
      S95_t1(i)%deltax=deltaMh_LEP
   endif
  enddo
   
  do i=lbound(S95_t2,dim=1),ubound(S95_t2,dim=1)
    S95_t2(i)%deltax=1.0D-7    !don't change this - the relevent part of code was taken out to improve efficiency
  enddo  
  
  ! finds the maximum deltaMh for the Tevatron tables
  ! will be used in theo_SM.f90 to work out which SM para need to be calculated
 
  max_tev_deltaMh= -1.0D0
  do i=lbound(S95_t1,dim=1),ubound(S95_t1,dim=1)
   if( (S95_t1(i)%expt.eq.'CDF') &
   .or.(S95_t1(i)%expt.eq.' D0') &
   .or.(S95_t1(i)%expt.eq.'TCB'))then
      if(S95_t1(i)%deltax.gt.max_tev_deltaMh)then
       max_tev_deltaMh=S95_t1(i)%deltax
      endif 
   endif
  enddo    
 
  max_lep_deltaMh= -1.0D0
  do i=lbound(S95_t1,dim=1),ubound(S95_t1,dim=1)
   if( S95_t1(i)%expt.eq.'LEP')then
      if(S95_t1(i)%deltax.gt.max_lep_deltaMh)then
       max_lep_deltaMh=S95_t1(i)%deltax
      endif 
   endif
  enddo 
  do i=lbound(S95_t2,dim=1),ubound(S95_t2,dim=1)
   if( S95_t2(i)%expt.eq.'LEP')then
      if(S95_t2(i)%deltax.gt.max_lep_deltaMh)then
       max_lep_deltaMh=S95_t2(i)%deltax
      endif 
   endif
  enddo 
 
  if(debug)write(*,*)'max_tev_deltaMh',max_tev_deltaMh
  if(debug)write(*,*)'max_lep_deltaMh',max_lep_deltaMh  

  TEVrange_min = TEVt1Mhmin  - max_tev_deltaMh
  TEVrange_max = TEVt1Mhmax  + max_tev_deltaMh
  LEPrange_min = LEPt12Mhmin - max_lep_deltaMh
  LEPrange_max = LEPt12Mhmax + max_lep_deltaMh

  !we need XS_SM_functions and BRfunctions to have a big enough range to cover the tables
  if(TEVrange_max.gt.XS_SM_functions_xmax)then
   write(*,*)TEVt1Mhmax,max_tev_deltaMh,XS_SM_functions_xmax
   stop'need to extend upper range of XS_SM_functions or reduce deltaMh_(LEP/TEV)'
  endif

  if(TEVrange_min.lt.XS_SM_functions_xmin)then
   stop'need to extend lower range of XS_SM_functions'
  endif
  
  if(   (TEVrange_max.gt.BRSMt1Mhmax).or.(LEPrange_max.gt.BRSMt1Mhmax))then
   stop'need to extend upper range of BRfunctions or reduce deltaMh_(LEP/TEV)'
  endif

  if(    TEVrange_min.lt.BRSMt1Mhmin     )then
   stop'need to extend lower range of BRfunctions'
  endif

  if(  LEPrange_min.lt.BRSMt1Mhmin )then
   if( LEPt12Mhmin .lt.BRSMt1Mhmin )then 
    stop'need to extend lower range of BRfunctions 1'
   else
    LEPrange_min = BRSMt1Mhmin
   endif
  endif


  do i=lbound(S95_t1,dim=1),ubound(S95_t1,dim=1)
   if( S95_t1(i)%charge.ne.0)then
    stop'Error in setup_S95tables 1: Have not implemented charged Higgs sector yet'
   endif
  enddo  

  do i=lbound(S95_t2,dim=1),ubound(S95_t2,dim=1)
   if( S95_t2(i)%charge.ne.0)then
    stop'Error in setup_S95tables 2: Have not implemented charged Higgs sector yet'
   endif
  enddo  

 end subroutine setup_S95tables	

 !******************************************************************	
 subroutine calcfact(proc,t,cfact,Mhi_av,Mhj_av,nc)
 !******************************************************************
 !for table type1, calls calcfact_t1
 !for table type2, calls calcfact_t2
 !****************************************************************** 
  use usefulbits, only : dataset,listprocesses  !internal
  implicit none	
  !--------------------------------------input
  type(dataset) :: t 
  type(listprocesses) :: proc
  !-----------------------------------output
  double precision :: cfact,Mhj_av,Mhi_av
  integer :: nc
  !-------------------------------------------	
 
  select case(proc%ttype)	
  case(1)
   call calcfact_t1(proc%tlist,proc%findj,t,cfact,Mhi_av,Mhj_av,nc)
  case(2)	
   call calcfact_t2(proc%tlist,proc%findj,proc%findi,t,cfact,Mhi_av,Mhj_av,nc)
  case default
   stop 'wrong input to function calcfact in module channels'
  end select		

 end subroutine calcfact

 !******************************************************************
 subroutine outputproc(proc,k,descrip,specific)
 !******************************************************************
 !for table type1, calls outputproc_t1
 !for table type2, calls outputproc_t2
 !if neither table applies, writes message
 !k is where output goes
 !specific=1 if the specific process should be prionted 
 !e.g. ee->h1Z->bbZ
 !whereas specific==0 if the generic process should be printed
 !e.g. ee->hiZ->bbZ  
 !****************************************************************** 
  use usefulbits, only : listprocesses !input
  implicit none
  !--------------------------------------input
  integer,intent(in) :: k,specific
  integer :: i,j
  type(listprocesses),intent(in) :: proc
  character(LEN=200):: descrip  
  !-------------------------------------------	   

  select case(specific)
  case(0)
   i=0
   j=0
  case(1)
   i=proc%findi
   j=proc%findj
  case default
  end select

  select case(proc%ttype)
  case(0)
   descrip='none of the processes apply'
  case(1)
   call outputproc_t1(proc%tlist,i,j,k,descrip)
  case(2)
   call outputproc_t2(proc%tlist,i,j,k,descrip) 
  case default
   stop 'wrong input to subroutine outputproc in module channels'
  end select	 	 
		
 end subroutine outputproc 

 !******************************************************************	
 subroutine check_against_bound(proc,t,fact,Mhi_av,Mhj_av,ratio,predobs)
 !******************************************************************
 !for table type1, calls interpolate_1D
 !for table type2, calls interpolate_2D
 !****************************************************************** 
  use usefulbits, only : dataset,listprocesses  !internal
  use interpolate
  implicit none
  !--------------------------------------input
  integer :: predobs
  double precision :: fact,Mhi_av,Mhj_av
  type(dataset) :: t  
  type(listprocesses) :: proc  
  !-------------------------------------output
  double precision :: ratio  
  !-----------------------------------internal
  double precision :: Mhi,Mhj,interpol
  !-------------------------------------------	
	
  Mhi=Mhi_av
  !Mhj=t%Mh(proc%findj)
  Mhj=Mhj_av		
  
  if(fact.ge.0.0D0)then 
   select case(proc%ttype)	
   case(1)
    call interpolate_1D(Mhj,S95_t1(proc%tlist),predobs,interpol)
   case(2)	
    call interpolate_2D(Mhi,Mhj,S95_t2(proc%tlist),predobs,interpol)
   case default
    write(*,*)'wrong input to subroutine check_against_bound in module channels'
    stop
   end select	
  endif	

  if(interpol.ge.0)then
   ratio=fact/interpol
  else
   ratio= -1.0D0
  endif

 end subroutine check_against_bound 

 !**********************************************************
 subroutine calcfact_t1(c,jj,t,cfact_t1,Mh_av_dummy,Mh_av,nc)
 !**********************************************************
 ! calculates fact for table type 1 
 ! Takes in to account how Standard Model-like parameter point is
 ! and whether there are any Higgs with slightly higher masses which
 ! can be combined with his result
 ! note: numerator and denominator are worked out separately
 !**********************************************************
  use usefulbits, only : dataset, nH, div, TEVt1Mhmax,TEVt1Mhmin, vvsmall
  use theory_BRfunctions
  use theory_XS_SM_functions 
    
  implicit none	
  !--------------------------------------input
  type(dataset) :: t
  integer :: c,jj
  !-----------------------------------output
  double precision :: cfact_t1,Mh_av,Mh_av_dummy
  integer :: nc
  !-------------------------------------------
  integer :: f,j
  double precision :: Mh_tot
  double precision :: BR_Hbb_SM_av,BR_Htautau_SM_av,BR_HWW_SM_av
  double precision :: BR_Hgaga_SM_av  
  double precision :: tev_XS_HW_SM_av,tev_XS_HZ_SM_av
  double precision :: tev_XS_H_SM_av
  !double precision :: tev_XS_H_SM_9674_av,tev_XS_H_SM_9713_av
  double precision :: tev_XS_Hb_SM_av,tev_XS_vbf_SM_av
  double precision :: BR_Zll,BR_Znunu,BR_Wlnu 
  double precision :: BR_Whad,BR_Zhad  
  double precision,allocatable :: mass(:),fact(:)
  logical :: TevTable,TevTableOutOfRange
  integer,allocatable :: SMlike(:)
  integer :: np !number of particles

  !source: PDG (Yao et al. J Phys G 33 (2006)) 
  BR_Zll=(3.363d0+3.366d0)/1.0D2     !BR_Zll = sum(l=e,mu), BR(Z ->l+ l-)
  BR_Znunu=20d0/1.0D2                !BR_Znunu = BR(Z ->nu_l nu_l-bar) ('invisible')
  BR_Wlnu=(10.75d0+10.57d0)/1.0D2    !BR_Wlnu = sum(l=e,mu),
                                     !BR(W+ ->l+ nu_l) = BR(W- ->l- + nu_l-bar)
  BR_Whad=67.6/1.0D2     
  BR_Zhad=69.91/1.0D2         
 
  np=nH
  allocate(mass(np),fact(np),SMlike(np))
  mass(:)=t%Mh(:)   

  fact= 0.0D0     
  cfact_t1=0.0D0    
  Mh_av_dummy=0.0D0  

  !now calculate numerator of 'fact' 
  do j=1,np    
  
   if(     (abs(mass(jj)-mass(j)).lt.S95_t1(c)%deltax) &
    & .and.(            mass(jj).le.mass(j)           )  )then
         
    select case(S95_t1(c)%id)
    case(142)
     fact(j)=t%lep%XS_hjZ_ratio(j)   *t%BR_hjbb(j) !notice that this is not absolute XS
    case(143)
     fact(j)=t%lep%XS_hjZ_ratio(j)   *t%BR_hjtautau(j) !notice that this is not absolute XS
    case(300)
     fact(j)=t%lep%XS_hjZ_ratio(j)   !notice that this is not absolute XS
    case(500)
     fact(j)=t%lep%XS_hjZ_ratio(j)   *t%BR_hjgaga(j)   !notice that this is not absolute XS
    case(8742,4493,9475,5482,5570,5876,1024,9889,3534)   
     fact(j)=t%tev%XS_hjZ_ratio(j)   *t%tev%XS_HZ_SM(j)  *t%BR_hjbb(j)        
    case(8958,5489,5624,5757,9236,3930)
     fact(j)=t%tev%XS_hj_ratio(j)*t%tev%XS_H_SM(j)   *t%BR_hjWW(j)
    case(8957,5472,9219,9463,9596,5828,1970,3493,5972,3155,5613,9868)  
     fact(j)=t%tev%XS_hjW_ratio(j)   *t%tev%XS_HW_SM(j)  *t%BR_hjbb(j)  
    case(5485,7307,5873)   
     fact(j)=t%tev%XS_hjW_ratio(j)   *t%tev%XS_HW_SM(j)  *t%BR_hjWW(j)       
    case(9071,2491,5740,5980,1014)
     fact(j)=t%tev%XS_hj_ratio(j)    *t%tev%XS_H_SM(j)   *t%BR_hjtautau(j) 
    case(8961,0598)   
     SMlike(j)=SMlikeness(j,S95_t1(c)%id,t) 
     select case(SMlike(j))    
     case(1)
      fact(j)=( t%tev%XS_hj_ratio(j)  * t%tev%XS_H_SM(j)  * t%BR_hjWW(j) * BR_Wlnu**2.0D0 &
      &  +   t%tev%XS_hjW_ratio(j) * t%tev%XS_HW_SM(j) * t%BR_hjWW(j) * BR_Wlnu**2.0D0 &
      &  +   t%tev%XS_hjZ_ratio(j) * t%tev%XS_HZ_SM(j) * t%BR_hjbb(j) * BR_Zll         &
      &  +   t%tev%XS_hjZ_ratio(j) * t%tev%XS_HZ_SM(j) * t%BR_hjbb(j) * BR_Znunu       &
      &  +   t%tev%XS_hjW_ratio(j) * t%tev%XS_HW_SM(j) * t%BR_hjbb(j) * BR_Wlnu        &
      &    )   
     case(0)
      fact(j)=t%tev%XS_hj_ratio(j) *t%tev%XS_H_SM(j)   *t%BR_hjWW(j)      
     case(-1)
      fact(j)= 0.0D0  
     case default  
      stop 'error in calcfact_t1 8961'     
     end select    
    case(9284,5503,5726)  
     fact(j)=t%tev%XS_hjb_ratio(j)    *t%tev%XS_Hb_SM(j)  *t%BR_hjbb(j)/0.9D0/2.0D0 
    case(1514,5601,5737)  
     fact(j)=(  t%tev%XS_hjZ_ratio(j) * t%tev%XS_HZ_SM(j)      &
      &  +   t%tev%XS_hjW_ratio(j) * t%tev%XS_HW_SM(j)      &
      &  +   t%tev%XS_hj_ratio(j)  * t%tev%XS_H_SM(j)       &
      &  +   t%tev%XS_vbf_ratio(j) * t%tev%XS_vbf_SM(j)     &
      &    )                  & 
      &  *   t%BR_hjgaga(j)      	  	
    case(5858,1887)  
     SMlike(j)=SMlikeness(j,S95_t1(c)%id,t)
     select case(SMlike(j))
     case(1)
     fact(j)=(  t%tev%XS_hjZ_ratio(j) * t%tev%XS_HZ_SM(j)      &
      &  +   t%tev%XS_hjW_ratio(j) * t%tev%XS_HW_SM(j)      &
      &  +   t%tev%XS_hj_ratio(j)  * t%tev%XS_H_SM(j)       &
      &  +   t%tev%XS_vbf_ratio(j) * t%tev%XS_vbf_SM(j)     &
      &    )                  & 
      &  *   t%BR_hjgaga(j)
     case(-1)
      fact(j)= 0.0D0   
     case default  
      stop 'error in calcfact_t1 5858'    
     end select  
    case(7081,9166,9483,5586,9642,1266,0432,9891)
     SMlike(j)=SMlikeness(j,S95_t1(c)%id,t)
     select case(SMlike(j))
     case(1)
      fact(j)=(  t%tev%XS_hjZ_ratio(j) * t%tev%XS_HZ_SM(j) * t%BR_hjbb(j) * BR_Znunu  &
       &     +   t%tev%XS_hjW_ratio(j) * t%tev%XS_HW_SM(j) * t%BR_hjbb(j) * BR_Wlnu   &
       &    )  
     case(-1)
      fact(j)= 0.0D0   
     case default  
      stop 'error in calcfact_t1 9166'    
     end select
    case(9248)
     SMlike(j)=SMlikeness(j,S95_t1(c)%id,t)
     select case(SMlike(j))
     case(1) 
      fact(j)=(  t%tev%XS_hjZ_ratio(j) * t%tev%XS_HZ_SM(j) * BR_Zhad &
       &  +   t%tev%XS_hjW_ratio(j) * t%tev%XS_HW_SM(j) * BR_Whad &
       &  +   t%tev%XS_hj_ratio(j)  * t%tev%XS_H_SM(j)       &
       &  +   t%tev%XS_vbf_ratio(j) * t%tev%XS_vbf_SM(j)     &
       &    )                  & 
       &  *   t%BR_hjtautau(j)   
     case(-1)
      fact(j)= 0.0D0      
     case default 
      stop 'error in calcfact_t1 9248'     
     end select     
    case(9290)
     SMlike(j)=SMlikeness(j,S95_t1(c)%id,t)
     select case(SMlike(j))
     case(1) 
      fact(j)=( t%tev%XS_hj_ratio(j)  * t%tev%XS_H_SM(j)  * t%BR_hjWW(j) * BR_Wlnu**2.0D0 &
      &  +   t%tev%XS_hjW_ratio(j) * t%tev%XS_HW_SM(j) * t%BR_hjWW(j) * BR_Wlnu**2.0D0 &
      &  +   t%tev%XS_hjZ_ratio(j) * t%tev%XS_HZ_SM(j) * t%BR_hjbb(j) * BR_Zll         &
      &  +   t%tev%XS_hjZ_ratio(j) * t%tev%XS_HZ_SM(j) * t%BR_hjbb(j) * BR_Znunu       &
      &  +   t%tev%XS_hjW_ratio(j) * t%tev%XS_HW_SM(j) * t%BR_hjbb(j) * BR_Wlnu        &      
      &  +  (t%tev%XS_hjZ_ratio(j) * t%tev%XS_HZ_SM(j)      &
      &  +   t%tev%XS_hjW_ratio(j) * t%tev%XS_HW_SM(j)      &
      &  +   t%tev%XS_hj_ratio(j)  * t%tev%XS_H_SM(j)       &
      &  +   t%tev%XS_vbf_ratio(j) * t%tev%XS_vbf_SM(j)     &
      &     )*t%BR_hjgaga(j)                                &
      &  +  (t%tev%XS_hjZ_ratio(j) * t%tev%XS_HZ_SM(j) * BR_Zhad &
      &  +   t%tev%XS_hjW_ratio(j) * t%tev%XS_HW_SM(j) * BR_Whad &
      &  +   t%tev%XS_hj_ratio(j)  * t%tev%XS_H_SM(j)       &
      &  +   t%tev%XS_vbf_ratio(j) * t%tev%XS_vbf_SM(j)     &
      &     )*t%BR_hjtautau(j)                              &
      &    )
     case(0)
      fact(j)=t%tev%XS_hj_ratio(j) *t%tev%XS_H_SM(j)   *t%BR_hjWW(j)    
     case(-1) 
      fact(j)= 0.0D0
     case default  
      stop 'error in calcfact_t1 9290' 
     end select 
    case(5984,9714)
     SMlike(j)=SMlikeness(j,S95_t1(c)%id,t)
     select case(SMlike(j))
     case(1) 
      fact(j)=( t%tev%XS_hj_ratio(j)  * t%tev%XS_H_SM(j)   &
      &     +   t%tev%XS_hjW_ratio(j) * t%tev%XS_HW_SM(j)  &   
      &     +   t%tev%XS_hjZ_ratio(j) * t%tev%XS_HZ_SM(j)  &
      &     +   t%tev%XS_vbf_ratio(j) * t%tev%XS_vbf_SM(j) &
      &    )* t%BR_hjWW(j)
     case(-1) 
      fact(j)= 0.0D0
     case default  
      stop 'error in calcfact_t1 5984' 
     end select
    case(3556)
     fact(j)=t%tev%XS_hjb_ratio(j)     *t%tev%XS_Hb_c2_SM(j)   *t%BR_hjbb(j)         	  	   	        
    case(9465,5871,9022,0710,9887)
     SMlike(j)=SMlikeness(j,S95_t1(c)%id,t)
     select case(SMlike(j))
     case(1)  
      fact(j)=(  t%tev%XS_hj_ratio(j)  * t%tev%XS_H_SM(j)   &
       &     +   t%tev%XS_hjW_ratio(j) * t%tev%XS_HW_SM(j)  &  
       &     +   t%tev%XS_vbf_ratio(j) * t%tev%XS_vbf_SM(j) &      
       &     +   t%tev%XS_hjZ_ratio(j) * t%tev%XS_HZ_SM(j)  &
       &     )  * t%BR_hjWW(j) 
     case(0)
      fact(j)=t%tev%XS_hj_ratio(j)*t%tev%XS_H_SM(j)   *t%BR_hjWW(j)  
     case(-1)
      fact(j)= 0.0D0   
     case default  
      stop 'error in calcfact_t1 9465' 
     end select
    !case(9023)
    ! SMlike(j)=SMlikeness(j,S95_t1(c)%id,t)
    ! select case(SMlike(j))
    ! case(1)  
    !  fact(j)=(  t%tev%XS_hj_ratio(j)  * t%tev%XS_H_SM_9713(j)   &
    !   &     +   t%tev%XS_hjW_ratio(j) * t%tev%XS_HW_SM(j)  &  
    !   &     +   t%tev%XS_vbf_ratio(j) * t%tev%XS_vbf_SM(j) &      
    !   &     +   t%tev%XS_hjZ_ratio(j) * t%tev%XS_HZ_SM(j)  &
    !   &     )  * t%BR_hjWW(j) 
    ! case(0)
    !  fact(j)=t%tev%XS_hj_ratio(j)*t%tev%XS_H_SM_9713(j)   *t%BR_hjWW(j)  
    ! case(-1)
    !  fact(j)= 0.0D0   
    ! case default  
    !  stop 'error in calcfact_t1 (B)' 
    ! end select
    case(9674)
     SMlike(j)=SMlikeness(j,S95_t1(c)%id,t)
     select case(SMlike(j))
     case(1)  
      !b: no longer using t%tev%XS_H_SM_9674(deleted 3 occurences)
      fact(j)=( t%tev%XS_hj_ratio(j)  * t%tev%XS_H_SM(j)  * t%BR_hjWW(j) * BR_Wlnu**2.0D0 &
      &  +   t%tev%XS_hjW_ratio(j) * t%tev%XS_HW_SM(j) * t%BR_hjWW(j) * BR_Wlnu**2.0D0 &
      &  +   t%tev%XS_hjZ_ratio(j) * t%tev%XS_HZ_SM(j) * t%BR_hjbb(j) * BR_Zll         &
      &  +   t%tev%XS_hjZ_ratio(j) * t%tev%XS_HZ_SM(j) * t%BR_hjbb(j) * BR_Znunu       &
      &  +   t%tev%XS_hjW_ratio(j) * t%tev%XS_HW_SM(j) * t%BR_hjbb(j) * BR_Wlnu        & 
      &  +  (t%tev%XS_hjZ_ratio(j) * t%tev%XS_HZ_SM(j) * BR_Zhad &
      &  +   t%tev%XS_hjW_ratio(j) * t%tev%XS_HW_SM(j) * BR_Whad &
      &  +   t%tev%XS_hj_ratio(j)  * t%tev%XS_H_SM(j)       &
      &  +   t%tev%XS_vbf_ratio(j) * t%tev%XS_vbf_SM(j)     &
      &     )*t%BR_hjtautau(j)                              &
      &    )
     case(0)
      fact(j)=t%tev%XS_hj_ratio(j)*t%tev%XS_H_SM(j)   *t%BR_hjWW(j) 
     case(-1)
      fact(j)= 0.0D0   
     case default  
      stop 'error in calcfact_t1 9674' 
     end select
    case(9897)
     SMlike(j)=SMlikeness(j,S95_t1(c)%id,t)
     select case(SMlike(j))
     case(1)  
      fact(j)=( &
      &  +  (t%tev%XS_hjZ_ratio(j) * t%tev%XS_HZ_SM(j)   &
      &  +   t%tev%XS_hjW_ratio(j) * t%tev%XS_HW_SM(j)   &
      &  +   t%tev%XS_hj_ratio(j)  * t%tev%XS_H_SM(j)    &
      &  +   t%tev%XS_vbf_ratio(j) * t%tev%XS_vbf_SM(j)  &
      &     )*t%BR_hjWW(j) * BR_Wlnu**2.0D0              &
      &  +  (t%tev%XS_hjZ_ratio(j) * t%tev%XS_HZ_SM(j) * BR_Zhad &
      &  +   t%tev%XS_hjW_ratio(j) * t%tev%XS_HW_SM(j) * BR_Whad &
      &  +   t%tev%XS_hj_ratio(j)  * t%tev%XS_H_SM(j)            &
      &  +   t%tev%XS_vbf_ratio(j) * t%tev%XS_vbf_SM(j)          &
      &     )*t%BR_hjtautau(j)                                   &
      &  +  (t%tev%XS_hjZ_ratio(j) * t%tev%XS_HZ_SM(j) * BR_Zhad   &
      &  +   t%tev%XS_hjZ_ratio(j) * t%tev%XS_HZ_SM(j) * BR_Zll    &
      &  +   t%tev%XS_hjZ_ratio(j) * t%tev%XS_HZ_SM(j) * BR_Znunu  &
      &  +   t%tev%XS_hjW_ratio(j) * t%tev%XS_HW_SM(j) * BR_Wlnu   & 
      &  +   t%tev%XS_hjW_ratio(j) * t%tev%XS_HW_SM(j) * BR_Whad   &
      &     )*t%BR_hjbb(j)                                         &
      &    )
     case(0)
      fact(j)=t%tev%XS_hj_ratio(j)*t%tev%XS_H_SM(j)   *t%BR_hjWW(j) 
     case(-1)
      fact(j)= 0.0D0   
     case default  
      stop 'error in calcfact_t1 9868' 
     end select
    !case(9713)
    ! note this is just an approximation!
    ! if((mass(jj).lt.150.0D0).or.(mass(jj).gt.200.0D0))then
    !  !stop'9713: not put proper "fact" for this yet'
    !  fact(j)=0.0D0
    ! else
    !  fact(j)=t%tev%XS_hj_ratio(j)*t%tev%XS_H_SM_9713(j)   *t%BR_hjWW(j)
    ! endif
    case(0024,5985)
     fact(j)=t%tev%XS_hjb_ratio(j)     *t%tev%XS_Hb_c3_SM(j)   *t%BR_hjtautau(j) 
    case default
     stop 'wrong input to function calcfact_t1 in module S95tables'
    end select

   endif

  enddo

  !TevTable=True if it's a tevatron table we're looking at  
   TevTable=((S95_t1(c)%expt.eq.'CDF').or.(S95_t1(c)%expt.eq.' D0')) &
       & .or.(S95_t1(c)%expt.eq.'TCB')
 
  if(fact(jj).le.vvsmall)then!A  !Higgs jj doesn't contribute - wait until another call of this subroutine before
                               !looking at nearby masses
   Mh_av = mass(jj)
   nc=0
   cfact_t1=0.0D0   
  else!A
   !find Mh_av (only using higgs which have non-zero fact): 
   f=0  
   Mh_tot=0.0D0
   do j=1,np     
    if(   fact(j).gt.vvsmall  )then       
     f=f+1
     Mh_tot=Mh_tot+mass(j) 
    endif
   enddo 
   
   nc=f !f will always be > 0 because we've already made sure that fact(jj)>0.0D0
  
   Mh_av = Mh_tot/dble(nc)  
   !TevTableOutOfRange=True if it's a tevatron table we're looking at but mass isn't in the range of
   ! any of the tevatron tables (means there's no need to calculate BR_*_SM_av, tev_*_SM_av, so it might be out of range of
   ! the subroutines that calculate them)
   TevTableOutOfRange=.False.

   ! It's now Mh_av that gets looked up in the expt tables
   ! so no point calculating cfact_t1 if Mh_av out of the range of the expt tables
   ! but this condition MUST be more strict than the one used in theo_manip to calculate t%tev%XS_*_SM(j)
   if( (Mh_av.lt.TEVt1Mhmin).or.(Mh_av.gt.TEVt1Mhmax) )then
    TevTableOutOfRange=.True.     
   endif 
   
   if(.not.TevTable)then!B
    cfact_t1=sum(fact)
   elseif(TevTableOutOfRange)then!B
    cfact_t1=0.0D0
   else!B

    if(f.eq.1)then !have already calculated these in theo_manip to save time
     BR_Hbb_SM_av     = t%BR_Hbb_SM(jj)
     BR_Htautau_SM_av = t%BR_Htautau_SM(jj)
     BR_HWW_SM_av     = t%BR_HWW_SM(jj)
     BR_Hgaga_SM_av   = t%BR_Hgaga_SM(jj)
     tev_XS_HW_SM_av  = t%tev%XS_HW_SM(jj)
     tev_XS_HZ_SM_av  = t%tev%XS_HZ_SM(jj)
     tev_XS_H_SM_av   = t%tev%XS_H_SM(jj)
     tev_XS_Hb_SM_av  = t%tev%XS_Hb_SM(jj)
     tev_XS_vbf_SM_av = t%tev%XS_vbf_SM(jj)
     !tev_XS_H_SM_9713_av   = t%tev%XS_H_SM_9713(jj)
     !tev_XS_H_SM_9674_av   = t%tev%XS_H_SM_9674(jj)
    else   
     BR_Hbb_SM_av     = BRSM_Hbb(Mh_av)  
     BR_Htautau_SM_av = BRSM_Htautau(Mh_av)
     BR_HWW_SM_av     = BRSM_HWW(Mh_av)
     BR_Hgaga_SM_av   = BRSM_Hgaga(Mh_av)
     tev_XS_HW_SM_av  = XS_qq_HW_SM(Mh_av)
     tev_XS_HZ_SM_av  = XS_qq_HZ_SM(Mh_av)
     tev_XS_H_SM_av   = XS_gg_H_SM(Mh_av)+XS_bb_H_SM(Mh_av) 
     tev_XS_Hb_SM_av  = XS_bg_Hb_SM(Mh_av) 
     tev_XS_vbf_SM_av = XS_vbf_SM(Mh_av) 
     !tev_XS_H_SM_9713_av   = XS_gg_H_SM_9713(Mh_av)+XS_bb_H_SM(Mh_av)   
     !tev_XS_H_SM_9674_av   = XS_gg_H_SM_9674(Mh_av)+XS_bb_H_SM(Mh_av)      
    endif     
    
    ! now include denominator of 'fact'
    select case(S95_t1(c)%id)
    case(8742,5482,5570,4493,9475,5876,1024,9889,3534)   
     fact=    fact  /  tev_XS_HZ_SM_av  /  BR_Hbb_SM_av   
    case(8958,5489,5624,9236,3930,5757)
     do j=1,np
      fact(j)=div(fact(j)  /  tev_XS_H_SM_av   ,  BR_HWW_SM_av ,0.0D0,0.0D0)  
     enddo
    case(8957,5472,9219,9463,9596,5828,1970,3493,5972,3155,5613,9868)  
     fact=    fact  /  tev_XS_HW_SM_av  /  BR_Hbb_SM_av   
    case(8961,0598)     
     do j=1,np 
      select case(SMlike(j))    
      case(1)
       fact(j)=   fact(j)   &
       &   /( tev_XS_H_SM_av  * BR_HWW_SM_av * BR_Wlnu**2.0D0 & 
       &  +   tev_XS_HW_SM_av * BR_HWW_SM_av * BR_Wlnu**2.0D0 &
       &  +   tev_XS_HZ_SM_av * BR_Hbb_SM_av * BR_Zll         &
       &  +   tev_XS_HZ_SM_av * BR_Hbb_SM_av * BR_Znunu       & 
       &  +   tev_XS_HW_SM_av * BR_Hbb_SM_av * BR_Wlnu        &
       &    )     
      case(0)
       fact(j)=div(fact(j) /  tev_XS_H_SM_av   ,  BR_HWW_SM_av ,0.0D0,0.0D0) 
      end select
     enddo
    case(5503,9284,5726)  
     fact=     fact /  tev_XS_Hb_SM_av  
    case(7307,5873) 
     do j=1,np
      fact(j)= div(fact(j) /  tev_XS_HW_SM_av  ,  BR_HWW_SM_av ,0.0D0,0.0D0)              	  	   	        
     enddo
    case(7081,9166,9483,5586,9642,1266,0432,9891)
      fact=    fact  &
       &   /( tev_XS_HZ_SM_av * BR_Hbb_SM_av * BR_Znunu  & 
       &  +   tev_XS_HW_SM_av * BR_Hbb_SM_av * BR_Wlnu  &
       &    )   
    case(9248)
      fact=    fact &
       &  /(    tev_XS_HZ_SM_av * BR_Zhad &
       &  +     tev_XS_HW_SM_av * BR_Whad &
       &  +     tev_XS_H_SM_av      &
       &  +     tev_XS_vbf_SM_av    &
       &    )                 & 
       &  /     BR_Htautau_SM_av   
    case(9290)  
     do j=1,np 
      select case(SMlike(j))
      case(1) 
       fact(j)=   fact(j)                                               &
       &   /( tev_XS_H_SM_av  * BR_HWW_SM_av     * BR_Wlnu**2.0D0 & 
       &  +   tev_XS_HW_SM_av * BR_HWW_SM_av     * BR_Wlnu**2.0D0 &
       &  +   tev_XS_HZ_SM_av * BR_Hbb_SM_av     * BR_Zll         &
       &  +   tev_XS_HZ_SM_av * BR_Hbb_SM_av     * BR_Znunu       & 
       &  +   tev_XS_HW_SM_av * BR_Hbb_SM_av     * BR_Wlnu        &
       &  +  (tev_XS_HZ_SM_av            &
       &  +   tev_XS_HW_SM_av            &
       &  +   tev_XS_H_SM_av             &
       &  +   tev_XS_vbf_SM_av           &     
       &     )*BR_Hgaga_SM_av            &             
       &  +  (tev_XS_HZ_SM_av * BR_Zhad  &
       &  +   tev_XS_HW_SM_av * BR_Whad  &
       &  +   tev_XS_H_SM_av             &
       &  +   tev_XS_vbf_SM_av           &
       &     )*BR_Htautau_SM_av          &
       &    )
      case(0)
       fact(j)= div(fact(j) /  tev_XS_H_SM_av   ,  BR_HWW_SM_av ,0.0D0,0.0D0)     
      end select
     enddo 
    case(5984,9714)  
     do j=1,np 
      select case(SMlike(j))
      case(1) 
       fact(j)= div(fact(j)                &
       &   /( tev_XS_H_SM_av               &
       &  +   tev_XS_HW_SM_av              &
       &  +   tev_XS_HZ_SM_av              &
       &  +   tev_XS_vbf_SM_av             &
       &    ) ,  BR_HWW_SM_av ,0.0D0,0.0D0) 
      end select
     enddo 
    case(9465,5871,9022,0710,9887)  
     do j=1,np 
      select case(SMlike(j))
      case(1)
       fact(j)= div(fact(j)                &
       &   /( tev_XS_H_SM_av               &
       &  +   tev_XS_HW_SM_av              &
       &  +   tev_XS_vbf_SM_av             &
       &  +   tev_XS_HZ_SM_av              &
       &    ) ,  BR_HWW_SM_av ,0.0D0,0.0D0) 
      case(0)
       fact(j)= div(fact(j) /  tev_XS_H_SM_av   ,  BR_HWW_SM_av ,0.0D0,0.0D0)     
      end select
     enddo        
    !case(9023)  
    ! do j=1,np 
    !  select case(SMlike(j))
    !  case(1)
    !   fact(j)= div(fact(j)                &
    !   &   /( tev_XS_H_SM_9713_av          &
    !   &  +   tev_XS_HW_SM_av              &
    !   &  +   tev_XS_vbf_SM_av             &
    !   &  +   tev_XS_HZ_SM_av              &
    !   &    ) ,  BR_HWW_SM_av ,0.0D0,0.0D0) 
    !  case(0)
    !   fact(j)= div(fact(j) /  tev_XS_H_SM_9713_av   ,  BR_HWW_SM_av ,0.0D0,0.0D0)     
    !  end select
    ! enddo   
    case(9674)  
     do j=1,np 
      !b: no longer using tev_XS_H_SM_9674_av (deleted 3 occurences)
      select case(SMlike(j))
      case(1)
       fact(j)=  fact(j)                                               &
       &   /( tev_XS_H_SM_av  * BR_HWW_SM_av     * BR_Wlnu**2.0D0 & 
       &  +   tev_XS_HW_SM_av      * BR_HWW_SM_av     * BR_Wlnu**2.0D0 &
       &  +   tev_XS_HZ_SM_av      * BR_Hbb_SM_av     * BR_Zll         &
       &  +   tev_XS_HZ_SM_av      * BR_Hbb_SM_av     * BR_Znunu       & 
       &  +   tev_XS_HW_SM_av      * BR_Hbb_SM_av     * BR_Wlnu        &            
       &  +  (tev_XS_HZ_SM_av      * BR_Zhad  &
       &  +   tev_XS_HW_SM_av      * BR_Whad  &
       &  +   tev_XS_H_SM_av             &
       &  +   tev_XS_vbf_SM_av           &
       &     )*BR_Htautau_SM_av          &
       &    )
      case(0)
       fact(j)= div(fact(j) /  tev_XS_H_SM_av   ,  BR_HWW_SM_av ,0.0D0,0.0D0)     
      end select
     enddo 
    case(9897)  
     do j=1,np 
      select case(SMlike(j))
      case(1)
       fact(j)=  fact(j)  &
       &   /((tev_XS_HZ_SM_av          & 
       &  +   tev_XS_HW_SM_av          &
       &  +   tev_XS_H_SM_av           &
       &  +   tev_XS_vbf_SM_av         &
       &     )*BR_HWW_SM_av  * BR_Wlnu**2.0D0     &           
       &  +  (tev_XS_HZ_SM_av      * BR_Zhad  &
       &  +   tev_XS_HW_SM_av      * BR_Whad  &
       &  +   tev_XS_H_SM_av             &
       &  +   tev_XS_vbf_SM_av           &
       &     )*BR_Htautau_SM_av          &
       &  +  (tev_XS_HZ_SM_av     * BR_Zhad  &
       &  +   tev_XS_HZ_SM_av     * BR_Zll   &
       &  +   tev_XS_HZ_SM_av     * BR_Znunu & 
       &  +   tev_XS_HW_SM_av     * BR_Wlnu  & 
       &  +   tev_XS_HW_SM_av     * BR_Whad  &
       &     )*BR_Hbb_SM_av                  &
       &    )
      case(0)
       fact(j)= div(fact(j) /  tev_XS_H_SM_av   ,  BR_HWW_SM_av ,0.0D0,0.0D0)     
      end select
     enddo         
    case(5737)  
     fact=fact/( ( tev_XS_HZ_SM_av   &
      &  +         tev_XS_HW_SM_av   &
      &  +         tev_XS_H_SM_av    &
      &  +         tev_XS_vbf_SM_av  &
      &           ) * BR_Hgaga_SM_av &
      &         )	
    case(5858,1887) 
     do j=1,np
      fact(j)=fact(j)/( ( tev_XS_HZ_SM_av   &
       &  +         tev_XS_HW_SM_av   &
       &  +         tev_XS_H_SM_av    &
       &  +         tev_XS_vbf_SM_av  &
       &           ) * BR_Hgaga_SM_av &
       &         )	  
     enddo 
    !case(9713)
    ! do j=1,np
    !  fact(j)=div(fact(j)  /  tev_XS_H_SM_9713_av   ,  BR_HWW_SM_av ,0.0D0,0.0D0)  
    ! enddo
    case(5485,9071,2491,5601,1514,3556,5740,0024,5980,1014,5985)
    case default
     stop 'error calculating denom. in calcfact_t1'
    end select
   
    cfact_t1=sum(fact)

   endif!B
  endif!A

  deallocate(mass)
  deallocate(fact)
  deallocate(SMlike)

 end subroutine calcfact_t1  
 !**********************************************************	
 subroutine calcfact_t2(c,jj,ii,t,cfact_t2,Mhi_av,Mhj_av,nc)
 !**********************************************************
 !calculates fact for table type 2 
 !**********************************************************	
  use usefulbits, only : dataset,nH
  implicit none	
  !--------------------------------------input	
  type(dataset) :: t
  integer :: c,jj,ii
  !-----------------------------------output
  double precision :: cfact_t2,Mhi_av,Mhj_av
  integer :: nc
  !-------------------------------------------
  integer :: f,i,j 
  double precision :: fact

  Mhi_av=t%Mh(ii)
  Mhj_av=t%Mh(jj)	
  
  fact= 0.0D0	              
  cfact_t2=0.0D0
  f=0 
          
  j=jj
  i=ii
!  do j=1,nH
!   if (     (abs(t%Mh(jj)-t%Mh(j)).lt.S95_t2(c)%deltax) &
!    &  .and.(           t%Mh(jj).le.t%Mh(j)            ) )then
!    do i=1,nH

!     if(      (     (abs(t%Mh(ii)-t%Mh(i)).lt.S95_t2(c)%deltax)&
!       &       .and.(           t%Mh(ii).le.t%Mh(i)            ) ) &
!       & .and.(         t%Mh(j).ge.t%Mh(i)                       ) ) then 
     if(   t%Mh(j).ge.t%Mh(i)  ) then	

      f=f+1
  	
      select case(S95_t2(c)%id)
      case(150)     
       fact=test_appl(t%lep%XS_hjZ_ratio(j)*t%BR_hjhihi(j,i)*t%BR_hjbb(i)**2.0D0)!table 15
      case(160)  
       fact=test_appl(t%lep%XS_hjZ_ratio(j)*t%BR_hjhihi(j,i)*t%BR_hjtautau(i)**2.0D0)!table 16
      case(180)  
       fact=test_appl(t%lep%XS_hjhi_ratio(j,i)*t%BR_hjbb(j)*t%BR_hjbb(i))!table 18		   
      case(190)
       fact=test_appl(t%lep%XS_hjhi_ratio(j,i)*t%BR_hjtautau(j)*t%BR_hjtautau(i))!table 19
      case(200)  
       fact=test_appl(t%lep%XS_hjhi_ratio(j,i)*t%BR_hjhihi(j,i)*t%BR_hjbb(i)**3.0D0)!table 20
      case(210) 
       fact=test_appl(t%lep%XS_hjhi_ratio(j,i)*t%BR_hjhihi(j,i)*t%BR_hjtautau(i)**3.0D0)!table 21
      case(220)  
       fact=test_appl(t%lep%XS_hjZ_ratio(j)*t%BR_hjhihi(j,i)*t%BR_hjbb(i)*t%BR_hjtautau(i))!table 22
      case(230)  	    	   
       fact=test_appl(t%lep%XS_hjhi_ratio(j,i)*t%BR_hjbb(j)*t%BR_hjtautau(i))!table 23   	 
      case(240) 
       fact=test_appl(t%lep%XS_hjhi_ratio(j,i)*t%BR_hjtautau(j)*t%BR_hjbb(i))!table 24
      case default
       stop 'wrong input to function calcfact_t2 in module S95tables'
      end select 		   
     else
      fact= 0.0D0
     endif
    
     cfact_t2=cfact_t2+fact	
!    enddo
!   endif
!  enddo 
  
  nc=f

  !if(f.gt.1)stop'f.ne.1 in calcfact_t2: this has not yet been thoroughly checked'  
  !*****
  
  contains
  
  !********************************************************	
  function test_appl(x)  
  !********************************************************
  !if hi->hj+hj is needed, check it's kinematically possible  
  !********************************************************
   implicit none
   !--------------------------------------input
   double precision :: x
   !-----------------------------------function
   double precision :: test_appl
   !-------------------------------------------	
   if(S95_t2(c)%needs_Mh2_gt_2Mh1.and.(t%Mh(j).lt.2.0D0*t%Mh(i)))then
    test_appl=0.0D0	
   else
    test_appl=x
   endif
  end function test_appl
	
 end subroutine calcfact_t2		
	
 !********************************************************	
 subroutine outputproc_t1(tlistn,ii,jj,k,descrip)
 !********************************************************		
 ! uses information about the process to output a description
 ! for processes using table type 1
 ! note: at the moment, nH (and so ii and jj) needs to be 1 digit long i.e. nH<10
 !******************************************************** 
  implicit none
  !--------------------------------------input
  integer :: tlistn
  integer :: ii,jj,k  
  !-----------------------------------internal
  character(LEN=1) :: j,i
  character(LEN=45) :: label
  character(LEN=200):: descrip
  !-------------------------------------------
	   
  if((ii.ne.0).and.(jj.ne.0))then	
   write(i,'(I1)')ii 
   write(j,'(I1)')jj	
  else
   i='i'
   j='j'
  endif

  if(k.eq.21)then
   label=''  !no need to lable each line in Key.dat
  else
   label='('//trim(S95_t1(tlistn)%label)//')'
  endif  	  
  
  descrip=''   
  
  select case(S95_t1(tlistn)%id)
  case(142)
   descrip=' (ee)->(h'//j//')Z->(b b-bar)Z   '     //label   	    
  case(143)
   descrip=' (ee)->(h'//j//')Z->(tau tau)Z   ' //label
  case(300)
   descrip=' (ee)->(h'//j//')Z->(...)Z   '     //label   
  case(500)
   descrip=' (ee)->(h'//j//')Z->(gamma gamma)Z   '     //label  
  case(5482,5570,8742,4493,9475,5876,1024,9889,3534)  
   descrip=' (p p-bar)->Z(h'//j//')->l l (b b-bar)   '  //label  
  case(9236,3930,8958)  
   descrip=' (p p-bar)->h'//j//'->W W   '           //label    
  case(9219,9463,5472,8957,9596,5828,1970,3493,5972,3155,5613,9868)  
   descrip=' (p p-bar)->W(h'//j//')->l nu (b b-bar)   ' //label     
  case(5489)  
   descrip=' (p p-bar)->h'//j//'->W W->e mu   '     //label     
  case(5624,5757)  
   descrip=' (p p-bar)->h'//j//'->W W->l l   '      //label
  case(5485,5873)  
   descrip=' (p p-bar)->W(h'//j//')->W W W->l l nu nu   '         //label  
  case(9071,2491,5740,5980,1014)  
   descrip=' (p p-bar)->h'//j//'->tau tau   '       //label         
  case(8961,9465,9290,9713,9674,0598,9897)  
   descrip=' (p p-bar)->h'//j//'+... where h'//j//' is SM-like  ' //label        
  case(9284,5503,5726,3556)  
   descrip=' (p p-bar)->h'//j//'(b/b-bar)->(b b-bar) (b/b-bar)   '          //label  
  case(7307)  
   descrip=' (p p-bar)->W(h'//j//')->W W W   '      //label 
  case(5601,5737,1514)    
   descrip=' (p p-bar)->h'//j//'+...->gamma gamma+... '  //label 
  case(5858,1887)    
   descrip=' (p p-bar)->h'//j//'+...->gamma gamma+... where h'//j//' is SM-like  '  //label 
  case(7081,9166,9483,5586,9642,1266,0432,9891)    
   descrip=' (p p-bar)->V h'//j//'->V (b b-bar) +missing Et '       //label 
  case(9248)    
   descrip=' (p p-bar)->h'//j//'+...->tau tau +... where h'//j//' is SM-like  ' //label
  case(5871)  
   descrip=' (p p-bar)->h'//j//'+...->W W +... ->l l nu nu +... where h'//j//' is SM-like ' //label
  case(9022,9023,0710,9887)  
   descrip=' (p p-bar)->h'//j//'+...->W W +... where h'//j//' is SM-like ' //label 
  case(5984,9714)  
   descrip=' (p p-bar)->h'//j//'+...->W W+... where h'//j//' is SM-like ' //label 
  case(0024,5985)  
   descrip=' (p p-bar)->h'//j//'(b/b-bar)->(tau tau) (b/b-bar)   '          //label  
  case default
   stop 'wrong input to function outputproc_t1 in module S95tables (1)'  
  end select   
  	  				
 end subroutine outputproc_t1
 
 !********************************************************	
 subroutine outputproc_t2(tlistn,ii,jj,k,descrip)
 !********************************************************		
 ! uses information about the process to output a description
 ! for processes using table type 1
 ! note: at the moment, nH (and so ii and jj) needs to be 1 digit long i.e. nH<10
 !******************************************************** 
  implicit none
  !--------------------------------------input
  integer :: tlistn
  integer :: ii,jj,k
  !-----------------------------------internal
  character(LEN=1) :: j,i
  character(LEN=45) :: label
  character(LEN=200):: descrip
  !-------------------------------------------
		   
  if((ii.ne.0).and.(jj.ne.0))then	
   write(i,'(I1)')ii 
   write(j,'(I1)')jj	
  else
   i='i'
   j='j'
  endif   

  if(k.eq.21)then
   label=''  !no need to lable each line in Key.dat
  else
   label='('//trim(S95_t2(tlistn)%label)//')'
  endif
	   	    	
  select case(S95_t2(tlistn)%id)
  case(150)	
   descrip=' (ee)->(h'//j//'->h'//i//' h'//i//')Z->(b b b b)Z   '         //label      
  case(160)	
   descrip=' (ee)->(h'//j//'->h'//i//' h'//i//')Z->(tau tau tau tau)Z   ' //label     
  case(180)	
   descrip=' (ee)->(h'//j//' h'//i//')->(b b b b)   '                     //label
  case(190)	
   descrip=' (ee)->(h'//j//' h'//i//')->(tau tau tau tau)   '             //label
  case(200)	
   descrip=' (ee)->(h'//j//'->h'//i//' h'//i//')h'//i//'->(b b b b)b b   '//label
  case(210)	
   descrip=' (ee)->(h'//j//'->h'//i//' h'//i//')h'//i//'->(tau tau tau tau)tau tau   '//label
  case(220)	
   descrip=' (ee)->(h'//j//'->h'//i//' h'//i//')Z->(b b)(tau tau)Z   '    //label
  case(230)	
   descrip=' (ee)->(h'//j//'->b b)(h'//i//'->tau tau)   '                 //label
  case(240)	
   descrip=' (ee)->(h'//j//'->tau tau)(h'//i//'->b b)   '                 //label
  case default
   stop 'wrong input to function outputproc_t2 in module S95tables (2)' 
  end select  	
		
 end subroutine outputproc_t2
 !******************************************************************
 function SMlikeness(j,id,t)
 !***************************************************************** 
 ! Tests how Standard Model-like a parameter point is            
 ! 0 means Mhi.ge.MhSingleLim (treat as single channel)
 ! 1 passes the SM-like test and Mhi.lt.MhSingleLim
 ! -1 fails the SM-like test and Mhi.lt.MhSingleLim 
  use usefulbits, only : dataset,div
  implicit none	
  !--------------------------------------input	
  type(dataset) :: t 
  integer :: id,j
  !-----------------------------------internal 
  integer :: ns,nb 
  double precision,allocatable :: XS_rat(:), BR_rat(:)
  double precision :: MhSingleLim
  integer :: SMlikeness
  integer :: is,ib
  double precision :: s,b
  double precision,allocatable :: dsbys(:),dbbyb(:)

  select case(id)
  case(8961,0598)
   ns = 3; allocate(XS_rat(ns))
   nb = 2; allocate(BR_rat(nb))	
	  
   XS_rat(1) = t%tev%XS_hjW_ratio(j)
   XS_rat(2) = t%tev%XS_hj_ratio(j)  
   XS_rat(3) = t%tev%XS_hjZ_ratio(j) 
  
   BR_rat(1) =     t%BR_hjbb(j) / t%BR_Hbb_SM(j)
   BR_rat(2) = div(t%BR_hjWW(j) , t%BR_HWW_SM(j) ,0.0D0,1.0D9)   
   
   !(beware:ignores HW->WWW process,but that doesn't contribute much) 
   !MhSingleLim = 150.0D0 !i.e. can be treated as a single channel above 150.0D0
   MhSingleLim = 1.0D6 !i.e. can be treated as a single channel nowhere  
  case(9290)
   ns = 4; allocate(XS_rat(ns))
   nb = 4; allocate(BR_rat(nb))	
	  
   XS_rat(1) = t%tev%XS_hjW_ratio(j)
   XS_rat(2) = t%tev%XS_hj_ratio(j)  
   XS_rat(3) = t%tev%XS_hjZ_ratio(j) 
   XS_rat(4) = t%tev%XS_vbf_ratio(j) 
  
   BR_rat(1) =     t%BR_hjbb(j) / t%BR_Hbb_SM(j)
   BR_rat(2) = div(t%BR_hjWW(j) , t%BR_HWW_SM(j) ,0.0D0,1.0D9)   
   BR_rat(3) = t%BR_hjtautau(j) / t%BR_Htautau_SM(j)  
   BR_rat(4) =   t%BR_hjgaga(j) / t%BR_Hgaga_SM(j)
     
   !(beware:ignores HW->WWW process,but that doesn't contribute much)
   !MhSingleLim = 150.0D0 !i.e. can be treated as a single channel above 150.0D0 
   MhSingleLim = 1.0D6 !i.e. can be treated as a single channel nowhere  
  case(9674,9897)
   ns = 4; allocate(XS_rat(ns))
   nb = 3; allocate(BR_rat(nb))	
	  
   XS_rat(1) = t%tev%XS_hjW_ratio(j)
   XS_rat(2) = t%tev%XS_hj_ratio(j)  
   XS_rat(3) = t%tev%XS_hjZ_ratio(j) 
   XS_rat(4) = t%tev%XS_vbf_ratio(j) 
  
   BR_rat(1) =     t%BR_hjbb(j) / t%BR_Hbb_SM(j)
   BR_rat(2) = div(t%BR_hjWW(j) , t%BR_HWW_SM(j) ,0.0D0,1.0D9)   
   BR_rat(3) = t%BR_hjtautau(j) / t%BR_Htautau_SM(j)  
     
   !(beware:ignores HW->WWW process,but that doesn't contribute much)
   !MhSingleLim = 150.0D0 !i.e. can be treated as a single channel above 150.0D0 
   MhSingleLim = 1.0D6 !i.e. can be treated as a single channel nowhere
  case(7081,9166,9483,5586,9642,1266,0432,9891)
   ns = 2; allocate(XS_rat(ns))
   nb = 1; allocate(BR_rat(nb))	
	  
   XS_rat(1) = t%tev%XS_hjW_ratio(j)
   XS_rat(2) = t%tev%XS_hjZ_ratio(j)  
  
   BR_rat(1) = t%BR_hjbb(j) / t%BR_Hbb_SM(j)
       
   MhSingleLim = 1.0D6 !i.e. can be treated as a single channel nowhere    
  
  case(9248)
   ns = 4; allocate(XS_rat(ns))
   nb = 1; allocate(BR_rat(nb))	
	  
   XS_rat(1) = t%tev%XS_hjW_ratio(j)
   XS_rat(2) = t%tev%XS_hj_ratio(j)  
   XS_rat(3) = t%tev%XS_hjZ_ratio(j)  
   XS_rat(4) = t%tev%XS_vbf_ratio(j)
  
   BR_rat(1) = t%BR_hjtautau(j) / t%BR_Htautau_SM(j)
      
   MhSingleLim = 1.0D6 !i.e. can be treated as a single channel nowhere

  case(5858,1887)
   ns = 4; allocate(XS_rat(ns))
   nb = 1; allocate(BR_rat(nb))	
	  
   XS_rat(1) = t%tev%XS_hjW_ratio(j)
   XS_rat(2) = t%tev%XS_hj_ratio(j)  
   XS_rat(3) = t%tev%XS_hjZ_ratio(j)  
   XS_rat(4) = t%tev%XS_vbf_ratio(j)
  
   BR_rat(1) = t%BR_hjgaga(j) / t%BR_Hgaga_SM(j)
      
   MhSingleLim = 1.0D6 !i.e. can be treated as a single channel nowhere

  case(9465,5871,9022,9023,0710,9887)
   ns = 4; allocate(XS_rat(ns))
   nb = 1; allocate(BR_rat(nb))	
	  
   XS_rat(1) = t%tev%XS_hjW_ratio(j)
   XS_rat(2) = t%tev%XS_hj_ratio(j)  
   XS_rat(3) = t%tev%XS_hjZ_ratio(j)  
   XS_rat(4) = t%tev%XS_vbf_ratio(j)
  
   BR_rat(1) = t%BR_hjWW(j) / t%BR_HWW_SM(j)   
   MhSingleLim = 1.0D6 !i.e. can be treated as a single channel nowhere

  case(5984,9714)
   ns = 4; allocate(XS_rat(ns))
   nb = 1; allocate(BR_rat(nb))	
	  
   XS_rat(1) = t%tev%XS_hjW_ratio(j)
   XS_rat(2) = t%tev%XS_hj_ratio(j)  
   XS_rat(3) = t%tev%XS_hjZ_ratio(j)  
   XS_rat(4) = t%tev%XS_vbf_ratio(j)
  
   BR_rat(1) = t%BR_hjWW(j) / t%BR_HWW_SM(j)   
   MhSingleLim = 1.0D6 !i.e. can be treated as a single channel nowhere
  case default
   stop 'error in subroutine SMlikeness'
  end select
        
  ns=ubound(XS_rat,dim=1)
  nb=ubound(BR_rat,dim=1)
  
  s=sum(XS_rat)/ns 
  b=sum(BR_rat)/nb    
    
  allocate(dsbys(ns))
  do is=1,ns    
   dsbys(is)= div((XS_rat(is) -s),s, 0.0D0,1.0D9)
  enddo
  
  allocate(dbbyb(nb))
  do ib=1,nb     
   dbbyb(ib)= div((BR_rat(ib) -b),b, 0.0D0,1.0D9) 
  enddo 

  if(t%Mh(j).ge.MhSingleLim)then ! treats Mh>MhSingleLim as single channel i.e. no SM-like test needed 
   SMlikeness= 0             
  else 			       
   SMlikeness= 1  !passes the SM-like test 
   do is=1,ns
    do ib=1,nb 
      if(abs( dsbys(is)+dbbyb(ib)+dsbys(is)*dbbyb(ib) ).gt.eps )then 
       SMlikeness= -1  !fails the SM-like test
      endif
    enddo 
   enddo
  endif 
  
  deallocate(dsbys)
  deallocate(dbbyb)  
  
  deallocate(XS_rat)
  deallocate(BR_rat) 
           
 end function SMlikeness

 !************************************************************	
 subroutine deallocate_S95tables
 !************************************************************
  implicit none
  !-----------------------------------internal
  integer x
  !-------------------------------------------
  do x=lbound(S95_t1,dim=1),ubound(S95_t1,dim=1)
   deallocate(S95_t1(x)%dat)
  enddo
  
  do x=lbound(S95_t2,dim=1),ubound(S95_t2,dim=1)
   deallocate(S95_t2(x)%dat)
  enddo  
  
  deallocate(S95_t1)
  deallocate(S95_t2)
 
 end subroutine deallocate_S95tables
 !***********************************************************
    	
end module S95tables
!************************************************************
