! This file is part of HiggsBounds
!  -KW
!******************************************************************
module input
!******************************************************************	
#ifdef NAGf90Fortran
 use F90_UNIX_ENV, only : iargc,getarg
 use F90_UNIX_IO, only : flush
#endif

 implicit none

 logical :: use_additional_file
 integer,parameter :: nHmax=9 ! do not change this without reading the comments in  
                              ! subroutine check_nH
 integer :: f_orig=15 ! n.b. debug_predratio.txt is 14, debug_channels.txt is 12 
                      ! (see HiggsBounds.f90 and HiggsBounds_subroutines.F90) 
 character(LEN=50),allocatable :: stem_array(:)

 contains


 !************************************************************	
 subroutine setup_input
 !************************************************************
 ! * if inputmethod='datfile' or 'website', finds whichanalyses,whichinput,nH
 ! * if inputmethod='datfile', finds infile1 (prefix for input/output filenames)
 ! * sets ndat (number of parameter points considered)
 ! * sets n_additional (number of additional data values for each parameter point)
 ! * allocates theo, g2, tevR
 !************************************************************
  use usefulbits, only : nH,ndat,n_additional,theo,g2,tevR, &
         &               inputmethod,whichinput, &
	 &               debug,infile1, &
	 &               allocate_tevextras_parts,allocate_dataset_parts, &
	 &               allocate_sqcouplratio_parts
  use extra_bits_for_SLHA
  use theory_BRfunctions	
  implicit none
  !-----------------------------------internal	
  integer :: f,ios,cc,g
  character(LEN=50) :: stem
  !-------------------------------------------	

  select case(inputmethod)
  case('datfile')
   call getbasiccommandline!get whichanalyses,whichinput,nH 
   call getshortcommandline(infile1)! get infile1 
  case('website')
   call getbasiccommandline!get whichanalyses,whichinput,nH
   !don't need infile1
  case('subrout') 
   !nH are already set
   !haven't yet set whichanalyses,whichinput
   infile1=''
  case default
   stop'incorrect value for inputmethod'
  end select

  !getting ndat and n_additional...
  select case(inputmethod)
  case('datfile')   
   f=f_orig
   use_additional_file=.False.
   n_additional=1
   select case(whichinput)
   case('part','effC','hadr') 
 
    call fill_stem_array

    g=1
    stem=stem_array(g)
    f=f_orig+g

    open(f,file=trim(infile1)//trim(stem)//'.dat',status='old',action='read',iostat=ios)
    if(ios.ne.0)then
     call file_name_msg(f)
     write(*,*)'Check that <prefix> was specified correctly'
     write(*,*)'and that file exists.'
     call flush(6)
     stop'Problem opening file: see standard output for more info'
    endif

    ndat=getfilelength(f) ! number of data sets (i.e. lines) in input 
 
    if((ndat.le.0))stop 'error getting ndat from Mh_GammaTot.dat'   	
    close(f)

    g=ubound(stem_array,dim=1)
    stem=stem_array(g)
    f=f_orig+g
    !the last element in stem_array should be 'additional'
    if(trim(stem_array(g)).ne.'additional')stop'Error in subroutine setup_input (a)'

    open(f,file=trim(infile1)//trim(stem)//'.dat',status='old',action='read',iostat=ios)    
    if(ios.eq.0)then    
     cc=count_columns(f)
     if(cc.gt.1)then
      if(getfilelength(f).eq.ndat)then
       n_additional=cc-1 ; use_additional_file=.True.
      endif
     endif
    endif
    close(f)

   case('SLHA')
    ndat=1  ;  n_additional=2 
   case default
    stop'error in subroutine do_input (1a)'
   end select
  case('website','subrout')
   ndat=1   ;  n_additional=1
  case default
   stop'error in subroutine do_input (1b)'
  end select
  !...finished getting ndat and n_additional

  if(debug)then
   write(*,*)'nH=',nH
   write(*,*)'ndat=',ndat
   write(*,*)'n_additional=',n_additional
  endif

  allocate(theo(ndat))	
  call allocate_dataset_parts(theo,n_additional,nH)  		  
  
  allocate(g2(ndat)) 
  call allocate_sqcouplratio_parts(g2,nH)
  
  allocate(tevR(ndat)) 
  call allocate_tevextras_parts(tevR,nH)  

 end subroutine setup_input

!************************************************************	
 subroutine do_input
 ! only used for inputmethod='datfile' or 'website'
 ! determines what input is needed and calls the appropriate
 ! subroutines to get this input
 !************************************************************
  use usefulbits, only : ndat,inputmethod,whichinput, &
         &               theo,g2,tevR,infile1
  use extra_bits_for_web, only : getlongcommandline2of2, &
         &                       getlongercommandline2of2         	  
  use extra_bits_for_SLHA
  use theory_BRfunctions	
  implicit none
  !-----------------------------------internal	
  integer :: jj,n
  character(LEN=50) :: stem
  logical :: webdebugmode
  !-------------------------------------------	

  
  select case(inputmethod)
  case('datfile')   
   select case(whichinput)
   case('part','effC','hadr')

    do n=1,ubound(stem_array,dim=1)
      call readthefile(n) 
    enddo
    deallocate(stem_array)

   case('SLHA')
    do jj=1,ndat
      call getSLHAdata(jj,theo(jj),tevR(jj),infile1) 
    enddo      

   case default
    stop'error in subroutine do_input (1)'
   end select

  case('website')   
   n=1
   select case(whichinput)	
   case('part','hadr') 
    call getlongercommandline2of2(whichinput,theo(n),tevR(n),webdebugmode)
   case('effC') 
    call getlongcommandline2of2(theo(n)%Mh,theo(n)%Gammatot, &
                   &            g2(n),theo(n)%BR_hjhihi,webdebugmode)    
   case default
    stop'error in subroutine do_input (2)'
   end select
   if(webdebugmode) call test_input(n)

  case default
   stop'error in subroutine do_input (3)'
  end select    

 end subroutine do_input
 !************************************************************
 subroutine fill_stem_array
 ! lists files which are relevant for 
 ! whichinput = 'part',effC' or 'hadr'
 ! note that stem_array(ubound(stem_array,dim=1)) should be 'additional'
 !************************************************************
 use usefulbits, only : whichinput
 implicit none
 integer :: n,nt
 
   n=1
   select case(whichinput)
   case('part')
    nt=11
    allocate(stem_array(nt))
    stem_array(n)='MH_GammaTot'             ; n=n+1
    stem_array(n)='BR_2H'                   ; n=n+1  
    stem_array(n)='BR_1H'                   ; n=n+1 
    stem_array(n)='LEP_HZ_CS_ratios'        ; n=n+1 
    stem_array(n)='LEP_2H_CS_ratios'        ; n=n+1 
    stem_array(n)='TEV_H_0jet_partCS_ratios'; n=n+1 
    stem_array(n)='TEV_H_1jet_partCS_ratios'; n=n+1 
    stem_array(n)='TEV_H_vbf_hadCS_ratios'  ; n=n+1
    stem_array(n)='TEV_HW_partCS_ratios'    ; n=n+1
    stem_array(n)='TEV_HZ_partCS_ratios'    ; n=n+1
    stem_array(n)='additional'              ; n=n+1

   case('effC')
    nt=4
    allocate(stem_array(nt))
    stem_array(n)='MH_GammaTot'             ; n=n+1
    stem_array(n)='effC'                    ; n=n+1
    stem_array(n)='BR_2H'                   ; n=n+1
    stem_array(n)='additional'              ; n=n+1

   case('hadr')
    nt=7
    allocate(stem_array(nt))
    stem_array(n)='MH_GammaTot'             ; n=n+1
    stem_array(n)='BR_2H'                   ; n=n+1
    stem_array(n)='BR_1H'                   ; n=n+1
    stem_array(n)='LEP_HZ_CS_ratios'        ; n=n+1
    stem_array(n)='LEP_2H_CS_ratios'        ; n=n+1
    stem_array(n)='TEV_1H_hadCS_ratios'     ; n=n+1
    stem_array(n)='additional'              ; n=n+1
   case default
    stop'Error in subroutine fill_stem_array A'
   end select

   if(n.ne.nt+1)stop'Error in subroutine fill_stem_array B'

 end subroutine fill_stem_array
 !************************************************************
 function is_file_needed(stem, use_addit_file)
 ! for some options of whichanalyses, some file are not needed - this
 ! function decides
 !************************************************************  
 use usefulbits, only : whichanalyses
 implicit none
  !-----------------------------------function
  logical :: is_file_needed 
  !--------------------------------------input
  logical,intent(in) :: use_addit_file
  character(LEN=50),intent(in) :: stem
  !-------------------------------------------

  is_file_needed=.False.
  select case(trim(stem))
  case('MH_GammaTot','BR_1H','effC')
    is_file_needed=.True.
  case('BR_2H','LEP_2H_CS_ratios')
    select case(whichanalyses)
    case('onlyL','LandT','onlyP') ! involves some LEP type2 
     is_file_needed=.True.
    end select 
  case('LEP_HZ_CS_ratios')   
    select case(whichanalyses)
    case('onlyL','LandT','singH','onlyP') ! involves some LEP type1  
     is_file_needed=.True.
    end select 
  case('TEV_H_0jet_partCS_ratios','TEV_H_1jet_partCS_ratios','TEV_H_vbf_hadCS_ratios', &
     & 'TEV_HW_partCS_ratios',    'TEV_HZ_partCS_ratios',    'TEV_1H_hadCS_ratios') 
    select case(whichanalyses)
    case('onlyT','LandT','singH','onlyP') ! involves some TEV type1  
     is_file_needed=.True.
    end select
  case('additional')
    if(use_addit_file)is_file_needed=.True. 
  case default
    stop'problem in subroutine readthefile (1)'    
  end select

 end function is_file_needed

 !************************************************************	
 subroutine readthefile(n_stem)
 !************************************************************
 ! Opens and reads the input data from files
 ! checks the number of lines and columns in the file first
 ! First column is always line number. Last line is checked to make
 ! sure line number is correct
 !************************************************************ 
  use usefulbits, only : nH,theo,ndat,infile1,g2,tevR, &
                   &     n_additional
  implicit none  
  !--------------------------------------input
  integer,intent(in) :: n_stem
  !-----------------------------------internal
  character(LEN=50) :: stem
  integer :: jj,i,j,f,k,q,ios
  double precision :: nc
  logical :: needed 
  double precision, allocatable :: BR_hjhihi_in(:)	
  !-------------------------------------------	
  stem=stem_array(n_stem)  

  needed=is_file_needed(stem,use_additional_file)

  if(needed)then
   f=f_orig+n_stem
   open(f,file=trim(infile1)//trim(stem)//'.dat',status='old',action='read',iostat=ios)	

    if(ios.ne.0)then
     call file_name_msg(f)
     stop'problem opening file: see standard output for more info'
    endif

   if((ndat.ne.getfilelength(f)))then
     write(*,*)'wrong no. lines in input file'
     write(*,*)'It had',getfilelength(f),'lines'
     write(*,*)'but should have been',ndat
     call file_name_msg(f)
     stop 'error in input file length (see standard output for filename).'
   endif

   if(get_ncol(stem).ne.count_columns(f))then
     write(*,*)'wrong no. columns in input file'
     write(*,*)'It had',count_columns(f),'columns'
     write(*,*)'but should have been',get_ncol(stem)
     write(*,*)'including line ID number'
     call file_name_msg(f)
     stop 'error in input file format (see standard output for filename).'
   endif

   select case(trim(stem))
   case('MH_GammaTot')
     do jj=1,ndat     
       read(f,*)  nc,  (theo(jj)%Mh(i)              ,i=1,nH), &
	   &           (theo(jj)%Gammatot(i)        ,i=1,nH)
     enddo
   case('BR_2H')
     allocate(BR_hjhihi_in(nH**2-nH))    
     BR_hjhihi_in  =0.0D0  

     do jj=1,ndat      
      read(f,*)  nc,(BR_hjhihi_in(i),i=1,int(nH**2-nH))  

      k=0
      do j=1,nH
        do i=1,nH
         if(i.ne.j)then
	  k=k+1
	  theo(jj)%BR_hjhihi(j,i) = BR_hjhihi_in(k)         
         endif           
        enddo
      enddo    
	
     enddo
     deallocate(BR_hjhihi_in)	
   case('BR_1H')       
     do jj=1,ndat          
       read(f,*)    nc, (theo(jj)%BR_hjbb(i)         ,i=1,nH), &
	     &          (theo(jj)%BR_hjtautau(i)     ,i=1,nH), &
	     &          (theo(jj)%BR_hjWW(i)         ,i=1,nH), &
	     &          (theo(jj)%BR_hjgaga(i)       ,i=1,nH) 
     enddo 
   case('LEP_HZ_CS_ratios')                 
     do jj=1,ndat    
      read(f,*)  nc, (theo(jj)%lep%XS_hjZ_ratio(i),i=1,nH) 
     enddo	
   case('LEP_2H_CS_ratios')
     do jj=1,ndat                                   
      read(f,*)  nc, ((theo(jj)%lep%XS_hjhi_ratio(j,i),i=1,j),j=1,nH)   
     enddo 
   case('TEV_1H_hadCS_ratios')  
     do jj=1,ndat    
      read(f,*)  nc, (theo(jj)%tev%XS_hj_ratio(i),i=1,nH) , &
                  &  (theo(jj)%tev%XS_hjb_ratio(i),i=1,nH) , &
                  &  (theo(jj)%tev%XS_hjW_ratio(i),i=1,nH) , &
                  &  (theo(jj)%tev%XS_hjZ_ratio(i),i=1,nH) , &
                  &  (theo(jj)%tev%XS_vbf_ratio(i),i=1,nH)   
     enddo  
   case('effC')        
     do jj=1,ndat                                    
      read(f,*)   nc,  (g2(jj)%hjbb(i)                   ,i=1,nH), &
           &           (g2(jj)%hjtautau(i)               ,i=1,nH), &
	   &           (g2(jj)%hjWW(i)                   ,i=1,nH), &
	   &           (g2(jj)%hjZZ(i)                   ,i=1,nH), &
	   &           (g2(jj)%hjgaga(i)                 ,i=1,nH), &
           &           (g2(jj)%hjgg(i)                   ,i=1,nH), &
           &           (g2(jj)%hjhiZ_in(i)               ,i=1,nH*(nH-1)/2+nH) 
    enddo       
  case('TEV_H_0jet_partCS_ratios')
    do jj=1,ndat     
     read(f,*)  nc, (tevR(jj)%gg_hj(i)           ,i=1,nH), &
         &          (tevR(jj)%qq_hj(5,i)         ,i=1,nH)    
    enddo   
  case('TEV_H_1jet_partCS_ratios')	     
    do jj=1,ndat     
     read(f,*)  nc, (tevR(jj)%bg_hjb(i),i=1,nH)   
    enddo   
  case('TEV_H_vbf_hadCS_ratios')    
    do jj=1,ndat     
     read(f,*)  nc, (theo(jj)%tev%XS_vbf_ratio(i),i=1,nH) 
    enddo    
  case('TEV_HW_partCS_ratios')
    do jj=1,ndat     
     read(f,*)  nc, ((tevR(jj)%qq_hjWp(q,i)       ,i=1,nH), q=1,tevR(jj)%nq_hjWp), &
         &          ((tevR(jj)%qq_hjWm(q,i)       ,i=1,nH), q=1,tevR(jj)%nq_hjWm)  
    enddo      
  case('TEV_HZ_partCS_ratios')
    do jj=1,ndat     
     read(f,*)  nc,  ((tevR(jj)%qq_hjZ(q,i)        ,i=1,nH), q=1,tevR(jj)%nq_hjZ ) 
    enddo      
  case('additional')          
    do jj=1,ndat 
     read(f,*)  nc, (theo(jj)%additional(i),i=1,n_additional)     		
    enddo 
  case default
    stop'problem in subroutine readthefile (2)'
  end select 
  
  close(f)

  if(ndat.ne.nint(nc))then 
     write(*,*)'last line read in was not labled ndat'
     call file_name_msg(f)
     stop 'error in input file (see standard output for filename and more details).'    
  endif 
  
  endif
 end subroutine readthefile
 !************************************************************	
 subroutine getbasiccommandline
 !************************************************************
 ! finds nH,whichanalyses,whichinput from command line 
 !************************************************************ 
  !nb iargc and getarg are non-standard
  use usefulbits, only : nH,whichanalyses,whichinput,inputmethod	
  implicit none  
  !-----------------------------------internal
#ifndef NAGf90Fortran
  integer :: iargc
#endif
  character(LEN=100) :: temp
  character(LEN=5) :: nHtemp
  integer :: i
  integer :: number_args 
  logical :: wrong_args 
  !-------------------------------------------	
  number_args = IARGC() 
   
  if(inputmethod .eq. 'datfile')then
   wrong_args= (number_args .ne. 4)
  elseif(inputmethod .eq. 'website')then 
   wrong_args= (number_args .lt. 3)
  else
   write(*,*)'inputmethod=',inputmethod
   stop 'error in getbasiccommandline'
  endif

  if( wrong_args )then
   write(*,*) "Incorrect number of parameters given on command line"
   call command_line_how2
   stop "Error: command line entered incorrectly (see standard output for more info)"
  endif
	
  ! Read arguments into text strings.
  i=1
  
  temp=""
  call GETARG(i,temp)
  whichanalyses = ""
  whichanalyses = trim(temp)
  i=i+1

  temp=""
  call GETARG(i,temp)
  whichinput = ""
  whichinput = trim(temp)
  i=i+1

  temp=""
  call GETARG(i,temp)
  nHtemp = ""
  nHtemp = trim(temp)

  if(verify(nHtemp," 1234567890").gt.0)then ! checks that the string nHtemp just contains the characters " 1234567890"
   ! the function verify is standard in fortran 95
   write(*,*)"Incorrect nH given as input."
   write(*,*)'(you entered nH="'//trim(adjustl(nHtemp))//'")'
   write(*,*)'nH is the number of neutral Higgs bosons.'
   call command_line_how2
   stop "Error: command line entered incorrectly (see standard output for more info)"
  endif

  read(nHtemp,*) nH
   
  call check_nH
  call check_whichinput 
  call check_whichanalyses

 end subroutine getbasiccommandline
 !************************************************************	
 subroutine command_line_how2
 !************************************************************	
   write(*,*)'The correct syntax for the command line is:'  
   write(*,*)' ./HiggsBounds whichanalyses whichinput nH prefix'
   write(*,*)'e.g.'
   write(*,*)' ./HiggsBounds LandT part 3 mhmax'
   write(*,*)'See HiggsBounds manual for more details.' 
   call flush(6)
 end subroutine command_line_how2
 !************************************************************	
 subroutine check_whichinput
 !************************************************************	
  use usefulbits, only : nH,whichinput,inputmethod

  select case(whichinput)
  case('part','effC','hadr')
  case('SLHA')
   stop 'The option SLHA option is not yet properly documented or supported.'
   if(nH.ne.3)stop 'must have nH=3 with whichinput=SLHA'   
   if(inputmethod.ne.'datfile')stop 'must have inputmethod = datfile with whichinput=SLHA'  
  case default
   call command_line_how2
   write(*,*)'Error: This value of "whichinput" is not allowed.'
   write(*,*)'(you entered whichinput="'//trim(adjustl(whichinput))//'")'
   write(*,*)'Allowed values for "whichinput" are (see manual):'
   write(*,*)'part        masses, branching ratios, decay widths, LEP cross sections,'
   write(*,*)'            Tevatron partonic cross sections'
   write(*,*)'effC        effective coupling approx' 
   write(*,*)'hadr        masses, branching ratios, decay widths, LEP cross sections,'
   write(*,*)'            Tevatron hadronic cross sections'
   !write(*,*)'SLHA        SUSY Les Houches Accord'       
   call flush(6)                  
   stop 'error: input type selected incorrectly (see standard output for more info)'   
  end select 

 end subroutine check_whichinput 

 !************************************************************	
 subroutine check_nH
 !************************************************************	
  use usefulbits, only : nH

  if(nH.le.0)then              
   stop 'nH.le.0 not allowed (subroutine getbasiccommandline (a))'
  endif  

  if(nH>nHmax)then
   write(*,*)'nH>9 not allowed in this version'
   write(*,*)'(if you really need nH>9,' 
   write(*,*)'change nHmax in subroutine setup_input,'   	 
   write(*,*)'and adapt subroutine outputproc_t1/t2' 
   write(*,*)'and do_output, by changing the size of'
   write(*,*)'the character variables nHchar,i,j'
   write(*,*)'since LEN=1 will no longer be sufficient)'	 
   call flush(6)
   stop 'nH>9 not allowed (subroutine getbasiccommandline (b)) (see standard output for more info)'
  endif  

 end subroutine check_nH 
 
 !************************************************************	
 subroutine check_whichanalyses
 !************************************************************	
  use usefulbits, only : whichanalyses
 
  select case(whichanalyses)
  case('onlyL','onlyT','LandT','singH','onlyP')
  case default
   call command_line_how2
   write(*,*)'Error: This value of "whichanalyses" is not allowed' 
   write(*,*)'(you entered whichanalyses="'//trim(adjustl(whichanalyses))//'")'
   write(*,*)'Allowed values for "whichanalyses" are:'
   write(*,*)'onlyL         only LEP results used'
   write(*,*)'onlyT         only Tevatron results used'   
   write(*,*)'LandT         both LEP and Tevatron results used'
   write(*,*)'onlyP         LEP results used, plus any Tevatron results published on arXiv'
   write(*,*)'singH         tables involving 1 Higgs only'
   write(*,*)'              (i.e. no Higgs pair production or Br(hj->hihi))'        
   call flush(6)                  
   stop 'error: experiment selected incorrectly (see standard output for more info)'  
  end select

 end subroutine check_whichanalyses 

 !************************************************************	
 subroutine getshortcommandline(inf1)
 !************************************************************
 ! used if inputmethod='datfile'
 ! finds infile1 from command line 
 !************************************************************ 
  !nb iargc and getarg are non-standard	
  implicit none
  !--------------------------------------input
  character(LEN=*) :: inf1   
  !-----------------------------------internal
#ifndef NAGf90Fortran
  integer :: iargc
#endif
  character(LEN=100) :: temp
  integer :: i
  integer :: number_args
  !-------------------------------------------	
   
  number_args = IARGC() 
  
  if( number_args .ne. 4  )then
   stop "Incorrect number of parameters given (getshortcommandline)"
  endif

  ! Read last argument into text string.
  i=4
		
  temp=""
  call GETARG(i,temp)
  inf1 = ""
  inf1 = trim(temp)
  i=i+1
 
  write(*,*)"------"
  write(*,*)"From command line:"
  write(*,*)"prefix:",trim(inf1) 
  write(*,*)"------"
  call flush(6)

 end subroutine getshortcommandline		
		
 !****************************************************
 function getfilelength(fileid)
 !****************************************************
 ! calculates file length and checks for errors
 ! nb. files must end in a 'newline' character
 !**************************************************** 
  implicit none
  !--------------------------------------input 
  integer fileid   
  !-----------------------------------function 
  integer :: getfilelength
  !-----------------------------------internal	
  integer :: n,ios,m
  character(LEN=5) :: filechar
  character(LEN=20) :: sample
  !-------------------------------------------	
  
  write(filechar,'(I5)')fileid	

  !this will count the number of lines in the file, including the last one,
  ! even if it doesn't end in a newline character
  n = 0				
  do
   read(fileid,'(a)',iostat=ios) sample

   if(ios.lt.0)then
    exit
   elseif(ios.gt.0) then
    call file_name_msg(fileid)
    stop 'error in input file: see standard output'
   elseif(trim(adjustl(sample)).eq.'')then
    write(*,*)'No blank lines allowed in input file.' 
    call file_name_msg(fileid)
    stop 'no blank lines allowed in input file: see standard output'
   endif
   n = n + 1   		
  enddo		

  if(n.eq.0)stop'File is empty'
	
  getfilelength=n
  rewind(fileid)

  m = 0 ;  
  do     !this will count the number of lines which end in a newline character      
   read(fileid,*,iostat=ios)
   if(ios.lt.0) exit 
   m = m + 1
  enddo
  rewind(fileid)

  if(m.ne.n)then !checking that every line end in a newline character
    call file_name_msg(fileid)
    stop'Error: file needs to end with a newline character (see standard output for filename)'
  endif

 end function getfilelength

 !****************************************************
 subroutine file_name_msg(fileid)
 !****************************************************
 use usefulbits,only : infile1
 implicit none
 integer, intent(in) :: fileid
    write(*,*)'The problematic input file is called:' 
    write(*,*)'  '//trim(adjustl(infile1))//trim(adjustl(stem_array(fileid-f_orig)))//'.dat'
    call flush(6)
 end subroutine file_name_msg

 !****************************************************
 function count_columns(fileid)
 !****************************************************
 ! calculateshow many columns of numbers there are in file 
 ! with the id 'fileid'
 ! max line length= 1000 characters
 ! assumes objects in columns are separated by spaces 
 ! and that first line has the same
 ! number of columns as the rest of the file
 !****************************************************
  implicit none
  !--------------------------------------input
  integer, intent(in) :: fileid
  !-----------------------------------function
  integer :: count_columns
  !-----------------------------------internal
  integer :: n,x,a
  character(LEN=1000) :: teststr
  character(LEN=1000) :: test
  !-------------------------------------------
 
  n=0
  read(fileid,'(a)')teststr ! reads first line into a string 'teststr'

  teststr=adjustl(teststr) ! shifts characters left so that first char is not a space
  
  a=1
  do while (a.gt.0)  !replace all tab characters with spaces

   a=INDEX(teststr,'	')
   teststr=teststr(:a-1)//'   '//teststr(a+1:)

  enddo

  teststr=adjustl(teststr) ! shifts characters left so that first char is not a space

  do while (trim(adjustl(teststr)).ne.'') !while teststr is not made of spaces   
   n=n+1   
   x=INDEX(teststr,' ') !finds position of first space from left
   teststr=adjustl(teststr(x:)) !chops of everything left of the first space, then adjustl
  enddo  
  
  count_columns=n 

  rewind(fileid) 
 
 end function count_columns 
 !********************************************************
 subroutine test_input(n)
 ! prints out input (useful for testing)
 !********************************************************
  use usefulbits, only : nH, theo, g2, tevR, whichinput
  implicit none
  !--------------------------------------input
  integer :: n
  !-----------------------------------internal
  integer :: j, i, q, k
  !-------------------------------------------
  
  write(*,*)'**************INPUT**************' 
  select case(whichinput)
  case('effC')
   write(*,*)'nH',nH 
   write(*,*)'theo(n)%Mh'
   write(*,*)theo(n)%Mh
   write(*,*)'theo(n)%Gammatot'
   write(*,*)theo(n)%Gammatot
   write(*,*)'g2(n)%hjbb'
   write(*,*)g2(n)%hjbb
   write(*,*)'g2(n)%hjtautau'
   write(*,*)g2(n)%hjtautau
   write(*,*)'g2(n)%hjWW'
   write(*,*)g2(n)%hjWW
   write(*,*)'g2(n)%hjZZ'
   write(*,*)g2(n)%hjZZ
   write(*,*)'g2(n)%hjgaga'
   write(*,*)g2(n)%hjgaga
   write(*,*)'g2(n)%hjgg'
   write(*,*)g2(n)%hjgg 
   write(*,*)'g2(n)%hjhiZ_in'
   
   do j=1,nH 
    k=j*(j-1)/2
    write(*,*)(g2(n)%hjhiZ_in(i+k),i=1,j)    
   enddo
!yeah, this is an odd way of writing it out... to see that it works, uncomment this bit:
!   do j=1,6 
!    k=j*(j-1)/2
!    write(*,*)(i+k,i=1,j)    
!   enddo

   write(*,*)'theo(n)%BR_hjhihi'
   do j=1,nH
    write(*,*)(theo(n)%BR_hjhihi(j,i),i=1,nH)
    call flush(6)
   enddo

   call flush(6)

  case('part')
   write(*,*)'theo(n)Mh'
   write(*,*) theo(n)%Mh
   write(*,*)'theo(n)%lep%XS_hjZ_ratio'
   write(*,*) theo(n)%lep%XS_hjZ_ratio

   write(*,*) 'theo(n)%lep%XS_hjhi_ratio'  
   do j=1,nH
    write(*,*)(theo(n)%lep%XS_hjhi_ratio(j,i),i=1,nH)
   enddo 

   write(*,*)'theo(n)%tev%XS_vbf_ratio'
   write(*,*) theo(n)%tev%XS_vbf_ratio


   write(*,*)'tevR(n)%bg_hjb'
   write(*,*) tevR(n)%bg_hjb


   write(*,*)'tevR(n)%qq_hjWp and tevR(n)%qq_hjWm'
   write(*,*)(tevR(n)%qq_hjWp(1,i),i=1,nH)
   write(*,*)(tevR(n)%qq_hjWp(2,i),i=1,nH)
   write(*,*)(tevR(n)%qq_hjWm(1,i),i=1,nH)
   write(*,*)(tevR(n)%qq_hjWm(2,i),i=1,nH)
   write(*,*)'tevR(n)%gg_hj'
   write(*,*) tevR(n)%gg_hj
   write(*,*)'tevR(n)%bb_hj'
   write(*,*)(tevR(n)%qq_hj(5,i),i=1,nH)
   write(*,*)'tevR(n)%bg_hjb'
   write(*,*) tevR(n)%bg_hjb


   write(*,*) 'tevR(n)%qq_hjZ'
   do q=1, tevR(n)%nq_hjZ
    write(*,*)(tevR(n)%qq_hjZ(q,i),i=1,nH)   
   enddo

   write(*,*)'theo(n)%BR_hjbb'
   write(*,*) theo(n)%BR_hjbb
   write(*,*)'theo(n)%BR_hjtautau'
   write(*,*) theo(n)%BR_hjtautau  
   write(*,*)'theo(n)%BR_hjWW'
   write(*,*) theo(n)%BR_hjWW
   write(*,*)'theo(n)%BR_hjgaga'
   write(*,*) theo(n)%BR_hjgaga
   write(*,*)'theo(n)%BR_hjhihi'  
   do j=1,nH
    write(*,*)(theo(n)%BR_hjhihi(j,i),i=1,nH)   
   enddo

  case('hadr')

   write(*,*)'theo(n)Mh'
   write(*,*) theo(n)%Mh
   write(*,*)'theo(n)%lep%XS_hjZ_ratio'
   write(*,*) theo(n)%lep%XS_hjZ_ratio

   write(*,*) 'theo(n)%lep%XS_hjhi_ratio'  
   do j=1,nH
    write(*,*)(theo(n)%lep%XS_hjhi_ratio(j,i),i=1,nH)
   enddo 
 
   write(*,*)'theo(n)%tev%XS_hj_ratio'
   write(*,*) theo(n)%tev%XS_hj_ratio
   write(*,*)'theo(n)%tev%XS_hjb_ratio'
   write(*,*) theo(n)%tev%XS_hjb_ratio
   write(*,*)'theo(n)%tev%XS_hjW_ratio'
   write(*,*) theo(n)%tev%XS_hjW_ratio
   write(*,*)'theo(n)%tev%XS_hjZ_ratio'
   write(*,*) theo(n)%tev%XS_hjZ_ratio
   write(*,*)'theo(n)%tev%XS_vbf_ratio'
   write(*,*) theo(n)%tev%XS_vbf_ratio

   write(*,*)'theo(n)%BR_hjbb'
   write(*,*) theo(n)%BR_hjbb
   write(*,*)'theo(n)%BR_hjtautau'
   write(*,*) theo(n)%BR_hjtautau  
   write(*,*)'theo(n)%BR_hjWW'
   write(*,*) theo(n)%BR_hjWW
   write(*,*)'theo(n)%BR_hjgaga'
   write(*,*) theo(n)%BR_hjgaga

   write(*,*)'theo(n)%BR_hjhihi'  
   do j=1,nH
    write(*,*)(theo(n)%BR_hjhihi(j,i),i=1,nH)   
   enddo

  case default
   stop'error in subroutine test_input'
  end select

  write(*,*)'*********************************' 
 end subroutine test_input   
 !*****************************************************
 function get_ncol(stem)
 ! calculates how many columns  Higgsounds should expect 
 ! to find in file stem//'.dat'
  use usefulbits, only : nH,n_additional

  implicit none
  !--------------------------------------input
   character(LEN=50), intent(in) :: stem
  !-----------------------------------function
   integer :: get_ncol
  !-----------------------------------internal
   integer :: x
  !-------------------------------------------

   select case(trim(stem))
   case('MH_GammaTot')
    x=2*nH
   case('BR_2H')
    x=nH*(nH-1)
   case('BR_1H')
    x=4*nH
   case('LEP_HZ_CS_ratios') 
    x=1*nH
   case('LEP_2H_CS_ratios')
    x= (nH*(nH+1))/2        
   case('TEV_1H_hadCS_ratios')  
    x=5*nH
   case('effC')
    x=6*nH  + (nH*(nH+1))/2
   case('TEV_H_0jet_partCS_ratios')
    x=2*nH
   case('TEV_H_1jet_partCS_ratios')
    x=1*nH
   case('TEV_H_vbf_hadCS_ratios')
    x=1*nH
   case('TEV_HW_partCS_ratios')
    x=4*nH
   case('TEV_HZ_partCS_ratios')
    x=5*nH
   case('additional')   
    x=n_additional 
   case default 
    write(*,*)'stem=',stem
    call flush(6)
    stop 'error in function get_ncol'
   end select

   get_ncol= x + 1

 end function get_ncol
 !************************************************************

end module input
!****************************************************
